/************************************************************
 * Queue.h
 *  - sicherere Queue-Klasse
 ************************************************************/
#ifndef _QUEUE_H
#define _QUEUE_H

#include <deque>
#include <exception>

template <class T>
class Queue {
  protected:
    std::deque<T> c;        // Container fuer die Elemente

  public:
    /* Exception-Klasse fuer pop() bei leerer Queue
     */
    class ReadEmptyQueue : public std::exception {
      public:
        virtual const char* what() const throw() {
            return "read empty queue";
        }
    };
  
    // Anzahl der Elemente
    std::deque<T>::size_type size() const {
        return c.size();
    }

    // leer ?
    bool empty() const {
        return c.empty();
    }

    // Zugriff auf naechstes Element
    T& front () {
        if (c.empty()) {
            throw ReadEmptyQueue();
        }
        return c.front();
    }

    // Element einfuegen
    void push (const T& elem) {
        c.push_back(elem);
    }

    // Element auslesen
    T pop () {
        if (c.empty()) {
            throw ReadEmptyQueue();
        }
        T elem(c.front());
        c.pop_front();
        return elem;
    }
};

#endif /* _QUEUE_H */
